//Tool for Control the 1Wire Barometric Preassure Sensor (www.tm3d.de)
//
//Copyright (C) 2012  Tobias Mueller mail (at) tobynet.de
//
//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
// any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
//VERSION 1.1 Control the 1Wire Barometric Preassure Sensor 

#include "ow_bmp085.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int readBMP085_4s(int portnum, uchar *SerialNum,uchar block, short *s) {
	int rt=FALSE;
	uchar send_block[30],lastcrc8;
	int send_cnt,  i, loop=0;
	
	owSerialNum(portnum,SerialNum,FALSE);
	for (loop = 0; loop < 2; loop ++) {
		if (owAccess(portnum)) {
			//copy block in scratchpad
			if (!owWriteByte(portnum,0x47+block))
				continue;
			msDelay(100);
			// access the device
			if (owAccess(portnum))  {
				// create a block to send that reads the temperature
				// read scratchpad command
				send_cnt = 0;
				send_block[send_cnt++] = 0xBE;
				// now add the read bytes for data bytes and crc8
				for (i = 0; i < 10; i++) send_block[send_cnt++] = 0xFF;
				// now send the block
				if (owBlock(portnum,FALSE,send_block,send_cnt)) {
					// initialize the CRC8 
					setcrc8(portnum,0);
					// perform the CRC8 on the last 8 bytes of packet
					for (i = send_cnt - 10; i < send_cnt; i++)
						lastcrc8 = docrc8(portnum,send_block[i]);
					// verify CRC8 is correct
					if (lastcrc8 == 0x00) {
						// calculate the high-res temperature
						for(i=0;i<4;i++) {
							s[i]= ((short*) (send_block+2))[i];
						}
						/*for(i=0;i<10;i++) {
							printf("%i ",send_block[i]);
						}*/
						//printf("\n");
						// success
						rt = TRUE;
						break;
					} else printf("CRC_ERROR\n");
				}
			}
		}
	}
	return rt;
}


int readBMP085(int portnum, uchar *SerialNum, double *Press, double *Temp, uchar *stat,long delay) {
	int rt=FALSE;
	uchar send_block[30],lastcrc8;
	int send_cnt,  i, loop=0;
	long *p,*t;
	
	owSerialNum(portnum,SerialNum,FALSE);
	for (loop = 0; loop < 2; loop ++) {
		if (owAccess(portnum)) {
			// send the convert command and start power delivery
			if (!owWriteByte(portnum,0x44))
				continue;
			// sleep for Conversion
			msDelay(delay);
			// access the device
			if (owAccess(portnum))  {
				// create a block to send that reads the temperature
				// read scratchpad command
				send_cnt = 0;
				send_block[send_cnt++] = 0xBE;
				// now add the read bytes for data bytes and crc8
				for (i = 0; i < 10; i++) send_block[send_cnt++] = 0xFF;
				// now send the block
				if (owBlock(portnum,FALSE,send_block,send_cnt)) {
					// initialize the CRC8 
					setcrc8(portnum,0);
					// perform the CRC8 on the last 8 bytes of packet
					for (i = send_cnt - 10; i < send_cnt; i++)
						lastcrc8 = docrc8(portnum,send_block[i]);
					// verify CRC8 is correct
					if (lastcrc8 == 0x00) {
						// calculate the high-res temperature
						t=(long*) (send_block+2);
						p=(long*) (send_block+6);
						*Press=(double)(*p)/100.0;
						*Temp=(double)(*t)/10.0;
						// success
						*stat=send_block[1];
						rt = TRUE;
						if ((send_block[1]&8)!=0) {
							printf("Measurement in progress\n");
							msDelay(1000);
							continue;
						}
						break;
					} else printf("CRC_ERROR\n");
				}
			}
		}
	}
	return rt;
}

int writeBMP085(int portnum, uchar *SerialNum,uchar stat) {
	int rt=FALSE;
	uchar send_block[30];
	int send_cnt;
	
	owSerialNum(portnum,SerialNum,FALSE);
	if (owAccess(portnum)) {
		send_cnt = 0;
		send_block[send_cnt++] = 0x4E;					
		send_block[send_cnt++] = stat;		
		owBlock(portnum,FALSE,send_block,send_cnt);

	}
	return TRUE;
}

